(       Title:  Search-order extension library
         File:  order.fs
    Test File:  order-test.fs
     Log File:  order.log
      Version:  0.8.0
      Revised:  November 16, 2010
  Anthologist:  David N. Williams
      License:  Public Domain

Much of the expression of Forth word definitions and
specifications in this library is believed to be in the public
domain, due to history or inevitability.  Anton Ertl, Michael L.
Gassanenko, Bruce McFarling, Bill Muench, John Sadler, and David
N. Williams have released to the public domain any of their
contributions to the words that did not already have that
status.

The word definitions in this library are consolidated from
contributions to the comp.lang.forth thread:

  Newsgroups: comp.lang.forth
  Date: Fri, 22 Jan 2010 11:41:50 -0500
  Subject: >ORDER ORDER> ORDER@

http://groups.google.com/group/comp.lang.forth/browse_frm/thread/c26bb6faeaae145a

The library emphasizes the role of the ANS/ISO Forth search
order as a stack.  The test file is order-test.fs, which
requires otester.fs and ttester.fs.

WORDS
-----

  >ORDER  ORDER>  ORDER@  ORDER!
  -ORDER  +ORDER

The first four words are being considered for a proposal for
Forth 20xx.

The word >ORDER has been in gforth since version 0.5.0, released
in 2000.  Its definition below was a factor in Helge Horch's
word METHODS, part of his public domain HYPE oof package, as
early as May, 1998.  The equivalent of that and the definition
of ORDER> below appeared as reference implementations in the
documentation for John Sadler's Ficl sometime in the mid 90's,
named as >SEARCH and SEARCH>.

All of the first four names were advocated by Michael Gassanenko
in 2002.  Functional equivalents have been used by various
people over a period of years.  A bit more history for all six
words can be found at the comp.lang.forth link above.
)

[UNDEFINED] >ORDER [IF]
: >ORDER  ( wid -- ) ( o: -- wid )
(
Move the wid from the data stack to the order stack.  "to-order"
)
  >r get-order r> swap 1+ set-order ;
[THEN]

: ORDER>  ( o: wid -- ) ( -- wid )
(
Move the wid from the order stack to the data stack. 
"order-from"
)
  get-order swap >r 1- set-order r> ;

[UNDEFINED] DROPS [IF]
: drops  ( xn ... x1 n -- )
(
Synonym for DISCARD, DPANS94 A.16.6.2.0715.
)
  0 ?DO drop LOOP ;
[THEN]

: ORDER@  ( o: wid -- wid ) ( -- wid )
(
Put a copy of the topmost order stack wid on the data stack. 
"order-fetch"
)
  get-order over >r drops r> ;

: ORDER!  ( wid -- ) ( o: wid' -- wid )
(
Overwrite the topmost order stack wid with the data stack wid.
"order-store"
)
  previous >ORDER ;

\ Implementations for _-ORDER, -ORDER, and +ORDER initially
\ supplied by Bill Muench.

[UNDEFINED] -ROT [IF] : -ROT  rot rot ; [THEN]

: _-ORDER ( w n*wid n -- n'*wid w n' )
(
A factor in -ORDER.  Assume that n is nonnegative.  The input
wid sequence n*wid may or may not contain copies of w.  The
output n'*wid has any copies removed, and n' is n minus the
number removed.  The implementation uses the return stack to
invert the order of wids, and recursively removes them to build
the output on the data stack.
)
  dup IF
    1- swap >r  ( >> ) RECURSE ( << )  ( i*wid w i) ( R: wid')
    over r@ xor IF 1+ r> -rot EXIT THEN r> drop
  THEN ;

: -ORDER  ( wid -- )  ( o: i*x -- j*x )
(
Remove all copies of the wid from the order stack.  "minus-order"
)
  get-order _-ORDER nip set-order ;

: +ORDER  ( wid -- )  ( o: i*x -- j*x wid )
(
Move the wid from the data stack to the order stack, and remove
any order stack copies below it.  "plus-order"
)
  dup -ORDER >ORDER ;
